﻿Omniscience = LibStub("AceAddon-3.0"):NewAddon("Omniscience", "AceEvent-3.0", "AceConsole-3.0", "LibBars-1.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

local bars = LibStub("LibBars-1.0")
local media = LibStub("LibSharedMedia-3.0")
local bitband = _G.bit.band
local floor = _G.math.floor
local pairs = _G.pairs
local tostring = _G.tostring
local strsub = _G.string.sub
local barlist = {}

local playerGUID = UnitGUID("player")

local function GetLSMIndex(t, value)
	for k, v in pairs(media:List(t)) do
	if v == value then
		return k
		end
	end
	return nil
end

local options = {
		type = "group",
		name = "Omniscience",
		handler = Omniscience,
		childGroups = "tab",
		args = { 
			runtest = {
				type = "execute",
				name = "Show Test Bars",
				func = function() Omniscience:RunTest() end,
			},
			toggleanchors = {
				type = "execute",
				name = "Toggle Anchors",
				func = function() Omniscience:ToggleAnchors() end,
			},
			bars = {
				type = "group",
				name = "Bar Options",
				args = {
					texture = {
						type = "select",
						name = "Texture",
						values = media:List("statusbar"),
						get = function(info) return GetLSMIndex("statusbar", Omniscience.db.profile.texture) end,
						set = function(info, value) Omniscience.db.profile.texture = media:List("statusbar")[value]; Omniscience:UpdateAnchors() end,
					},
					font = {
						type = "select",
						name = "Font",
						values = media:List("font"),
						get = function(info) return GetLSMIndex("font", Omniscience.db.profile.font) end,
						set = function(info, value) Omniscience.db.profile.font = media:List("font")[value]; Omniscience:UpdateAnchors() end,
					},
					fontsize = {
						type = "range",
						name = "Font Size",
						min = 4,
						max = 30,
						step = 1,
						get = function() return Omniscience.db.profile.fontsize end,
						set = function(info, value) Omniscience.db.profile.fontsize = value; Omniscience:UpdateAnchors() end,
					},
				},
			},
		},
	}


AceConfig:RegisterOptionsTable("Omniscience", options)

function Omniscience:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("OmniscienceDB")
	self.db:RegisterDefaults({
		profile = {
			texture = "Ruben",
			font = "ABF",
			fontsize = 9,
			width = "250",
			height = "10",
			anchors = { 
				cd =   {name = "Cooldowns",     enabled = true, position = {}, colors = {r=0, g=0, b=1}},
				cast = {name = "Casts",         enabled = true, position = {}, colors = {r=1, g=0, b=0}},
				buff = {name = "Buffs",         enabled = true, position = {}, colors = {r=0, g=1, b=0}},
				cc =   {name = "Crowd Control", enabled = true, position = {}, colors = {r=1, g=1, b=0}},
			},
			map = {
				cd = "cd",
				cast = "cast",
				buff = "buff",
				cc = "cc",
			},
		},
	})		

        self:RegisterChatCommand("omniscience", function() LibStub("AceConfigDialog-3.0"):Open("Omniscience") end )
        optFrame = AceConfigDialog:AddToBlizOptions("Omniscience", "Omniscience")
        options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	for anchor in pairs(self.db.profile.anchors) do
		self[anchor] = self:NewBarGroup(self.db.profile.anchors[anchor].name, nil, self.db.profile.width, self.db.profile.height)
		self[anchor]:SetFlashPeriod(0)
		self[anchor]:ClearAllPoints()
		if not self.db.profile.anchors[anchor].position then self.db.profile.anchors[anchor].position = {} end

		if self.db.profile.anchors[anchor].position.p
			and self.db.profile.anchors[anchor].position.pr
			and self.db.profile.anchors[anchor].position.px
			and self.db.profile.anchors[anchor].position.py then
				self[anchor]:ClearAllPoints()
				self[anchor]:SetPoint(self.db.profile.anchors[anchor].position.p, UIParent, self.db.profile.anchors[anchor].position.pr, self.db.profile.anchors[anchor].position.px, self.db.profile.anchors[anchor].position.py)
		else
			self[anchor]:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
		end
		self[anchor].id = anchor
		self[anchor].RegisterCallback(self, "AnchorClicked")
		self[anchor]:HideAnchor()
	end
	self:UpdateAnchors()
end

function Omniscience:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	media.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateAnchors")  
	media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "UpdateAnchors")  
	self:UpdateAnchors()
end

function Omniscience:OnDisable()
	self:UnRegisterAllEvents()
	media.UnregisterCallback(self)
	for anchor in pairs(self.db.profile.anchors) do
		self[anchor].UnregisterAllCallbacks()
	end
end

function Omniscience:UpdateAnchors()
	for anchor in pairs(self.db.profile.anchors) do
		self[anchor]:SetTexture(media:Fetch("statusbar", self.db.profile.texture))
		self[anchor]:SetFont(media:Fetch("font", self.db.profile.font, self.db.profile.fontsize))
		self[anchor]:SetColorAt(0.00, self.db.profile.anchors[anchor].colors.r, self.db.profile.anchors[anchor].colors.g, self.db.profile.anchors[anchor].colors.b, 1)
		self[anchor]:SetColorAt(1.00, self.db.profile.anchors[anchor].colors.r, self.db.profile.anchors[anchor].colors.g, self.db.profile.anchors[anchor].colors.b, 1)
		self[anchor]:SetScale(0.75)
	end
end

function Omniscience:AnchorClicked(callback, group, button)
	local anchor = group.id
	self:SavePosition(anchor)
	if button == "RightButton" then
		group:HideAnchor()
		group:Lock()
	end
end

function Omniscience:ToggleAnchors()
	for anchor in pairs(self.db.profile.anchors) do
		if self[anchor].button:IsVisible() then
			self[anchor]:HideAnchor()
			self[anchor]:Lock()
		else
			self[anchor]:ShowAnchor()
			self[anchor]:Unlock()
		end
	end
end

function Omniscience:SavePosition(anchor)
	local p,_,pr,px,py = self[anchor]:GetPoint()
	self.db.profile.anchors[anchor].position.p = p
	self.db.profile.anchors[anchor].position.pr = pr
	self.db.profile.anchors[anchor].position.px = px
	self.db.profile.anchors[anchor].position.py = py
end

function Omniscience:RunTest()
	for anchor in pairs(self.db.profile.anchors) do
		local icon = select(3, GetSpellInfo(43308))
		self[anchor]:NewTimerBar("test-"..anchor, self.db.profile.anchors[anchor].name, 30, nil, icon)
		self[anchor]:SortBars()
	end
end

local function CheckAffiliation(flags, match)
	local result = (bitband(flags, match) ~= 0)
	return result or nil
end

function Omniscience:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, spellID, spellName, spellFlags, ...)
	if eventType == "SPELL_INTERRUPT" then
		if not self.spells["SPELL_CAST_START"][spellID] then return end
		local group = self.spells["SPELL_CAST_START"][spellID].group
		local anchor = self.db.profile.map[group]
		local interrupted = select(1, ...)
		local name = "Omniscience-"..dstGUID.."-"..interrupted
		local bar = self[anchor]:GetBar(name)
		if bar then bar:Fade() end
		self[anchor]:SortBars()
	return end

	if eventType == "SPELL_AURA_REMOVED" then
		if not self.spells["SPELL_AURA_APPLIED"][spellID] then return end
		local group = self.spells["SPELL_AURA_APPLIED"][spellID].group
		local anchor = self.db.profile.map[group]
		local name = "Omniscience-"..dstGUID.."-"..spellID
		local bar = self[anchor]:GetBar(name)
		if bar then bar:Fade() end
		self[anchor]:SortBars()
	return end

	if eventType == "UNIT_DIED" then
		for anchor in pairs(self.db.profile.anchors) do
			if dstGUID == playerGUID then
				for bar in self[anchor]:GetBars() do
					if bar then bar:Fade() end
				end
			else
				for bar in self[anchor]:GetBars() do
					if bar and bar.name and strfind(bar.name, dstGUID) then
						bar:Fade()
					end
				end
			end
			self[anchor]:SortBars()
		end
	return end

	if not self.spells[eventType] then return end
	if not self.spells[eventType][spellID] then return end
	local flags, match, actor, actorGUID
	if self.spells[eventType][spellID].srcflags then
		flags, match = srcFlags, self.spells[eventType][spellID].srcflags
		actor, actorGUID = srcName, srcGUID
	elseif self.spells[eventType][spellID].dstflags then
		flags, match = dstFlags, self.spells[eventType][spellID].dstflags
		actor, actorGUID = dstName, dstGUID
	end
	if not actor or not actorGUID or not flags or not match then return end
	if not CheckAffiliation(flags, match) then return end

	local name = "Omniscience-"..actorGUID.."-"..spellID
	local value = self.spells[eventType][spellID].time
	local text = actor..": "..spellName
	local icon = select(3, GetSpellInfo(spellID)) or nil
	local group = self.spells[eventType][spellID].group
	local anchor = self.db.profile.map[group]
	self[anchor]:NewTimerBar(name, text, value, nil, icon)
	self[anchor]:SortBars()
end
